% A replication file for "Monetary Policy Surprises and Exchange Rate
% Behavior" by Gurkaynak, Kara, Kisacikoglu and Lee.

% This m file plots various impulse response functions in the paper. This 
% builds on full information and partial information rational expectations 
% solution packages in Dynare and requires Dynare to work.

clear;
clc;
close all;

%% Load parameters

% 1. Fixed parameters

% Common to both countries
gamma = 0.5;

% US
rhou_infl = 0;
rhou_r = 0; 
infltargbar = 0;
c2 = 0.99; 
c4 = 0.99; 

% EA
rhou_inflstar = 0;
rhou_rstar = 0;
infltargstarbar = 0;
c2star = 0.99; 
c4star = 0.99;

% 2. Estimated parameters 

% load result_noimpinf.mat;
% load result_impinf_r.mat;
% load result_impinf_infl_r.mat;
load result_impinf_infl_r_infltarg.mat;
% load result_impinf_rr_infl_r.mat;
% load result_impinf_rr_infl_r_infltarg.mat;
% load result_impinf_ytilde_rr_infl_r.mat;

% US
beta = xgmm3(1);
sigma = xgmm3(2);
phi = xgmm3(3);
theta = xgmm3(4);
rhoa = xgmm3(5);
rhor = xgmm3(6);
phipi = xgmm3(7);
phiy = xgmm3(8);
rhoinfltarg = xgmm3(9);
std_infl = xgmm3(19);
std_a = xgmm3(20);
std_r = xgmm3(21);
std_infltarg = xgmm3(22);
delta = ((1-theta)*(1-beta*theta))/theta;
kappa0 = gamma*(sigma-1);
kappa = sigma+phi-kappa0;
lambda = delta*kappa;
sigma0 = sigma-kappa0;

% EA
betastar = xgmm3(10);
sigmastar = xgmm3(11);
phistar = xgmm3(12);
thetastar = xgmm3(13);
rhoastar = xgmm3(14);
rhorstar = xgmm3(15); 
phipistar = xgmm3(16);
phiystar = xgmm3(17);
rhoinfltargstar = xgmm3(18);
std_inflstar = xgmm3(23);
std_astar = xgmm3(24);
std_rstar = xgmm3(25);
std_infltargstar = xgmm3(26);
deltastar = ((1-thetastar)*(1-betastar*thetastar))/thetastar;
kappa0star = (1-gamma)*(sigmastar-1);
kappastar = sigmastar+phistar-kappa0star;
lambdastar = deltastar*kappastar;
sigma0star = sigmastar-kappa0star;

save parameterfiletemp beta sigma phi theta gamma kappa0 kappa sigma0 delta lambda ...
    rhou_infl rhoa rhor rhou_r phipi phiy rhoinfltarg infltargbar c2 c4 ...
    betastar sigmastar phistar thetastar kappa0star kappastar sigma0star deltastar lambdastar ...
    rhou_inflstar rhoastar rhorstar rhou_rstar phipistar phiystar rhoinfltargstar infltargstarbar c2star c4star ...
    std_infl std_a std_r std_infltarg ...
    std_inflstar std_astar std_rstar std_infltargstar;

%% Solve

dynare NK_CGG02_impinf4.mod; % mod file needs to be adjusted when the information structure changes
impinf = 1; % 1 if there is imperfect information
irfpers = 100; % length of simulation
irfplot = 20; % length of irfs plotted
shockoption = 1; 
% US:
% 1 := inflation shock; 2 := productivity shock; 3 := nominal interest rate shock;
% 4 := inflation target shock;  
% EA:
% 5 := inflation shock; 6 := productivity shock; 7 := nominal interest rate shock;
% 8 := inflation target shock;

if impinf==0
    
    coefx = oo_.dr.ghx; % coefficients for state variables
    coefu = oo_.dr.ghu; % coefficients for shocks
    statevarind = oo_.dr.inv_order_var(oo_.dr.state_var'); % state variable index
    irfmat = zeros(size(coefx,1),irfpers+1); 
    stderr = diag(M_.Sigma_e^0.5);
    eps = zeros(size(coefu,2),irfpers+1);
    eps(shockoption,1) = stderr(shockoption,1);
        
    irfmat(:,1) = coefu*eps(:,1);
    for jj = 2:irfpers+1
        irfmat(:,jj) = coefx*irfmat(statevarind,jj-1)+coefu*eps(:,jj);
    end
    
    varnum = [1 3 5 6 12 14 16 17]';
    varnumdr=oo_.dr.inv_order_var(varnum); % ytilde y infl r (in the declaration order)
    figure;
    for kk = 1:8
        subplot(3,3,kk)
        plot((1:irfplot),round(irfmat(varnumdr(kk),1:irfplot),10),'b-','LineWidth',2);
        hold on;
        plot((1:irfplot),0*ones(irfplot,1),'r-','LineWidth',2);
        xlim([1 irfplot]);
        ylim([min(0,min(irfmat(varnumdr(kk),1:irfplot))) max(0,max(irfmat(varnumdr(kk),1:irfplot)))]); 
        title([deblank(M_.endo_names(varnum(kk),:))],'FontSize',25);
        x12 = get(gca,'XAxis');
        set(x12,'FontSize',22);
        y12 = get(gca,'YAxis');
        set(y12,'FontSize',22);
    end
    
    tot = irfmat(oo_.dr.inv_order_var(3),1:irfplot)-irfmat(oo_.dr.inv_order_var(14),1:irfplot); % terms of trade
    domprice = cumsum(irfmat(oo_.dr.inv_order_var(5),1:irfplot)); % initial price level normalized to zero
    forprice = cumsum(irfmat(oo_.dr.inv_order_var(16),1:irfplot)); % initial price level normalized to zero
    nomexc = tot+domprice-forprice; % nominal exchange rate
    
    subplot(3,3,9)
    plot((1:irfplot),round(nomexc,10),'b-','LineWidth',2);
    hold on;
    plot((1:irfplot),0*ones(irfplot,1),'r-','LineWidth',2);
    xlim([1 irfplot]);
    ylim([min(0,min(nomexc)) max(0,max(nomexc))]); 
    title('e','FontSize',25);
    x12 = get(gca,'XAxis');
    set(x12,'FontSize',22);
    y12 = get(gca,'YAxis');
    set(y12,'FontSize',22);
    
    % Nominal term structure - US
    nomint = irfmat(oo_.dr.inv_order_var(6),:);
    nomint2 = zeros(1,20);
    nomint4 = zeros(1,20);
    nomint8 = zeros(1,20);
    nomint20 = zeros(1,20);
    nomint40 = zeros(1,20);
    for ii = 1:min(irfplot,irfpers-40)
        nomint2(1,ii) = (1/2)*(sum(nomint(1,ii:ii+1)));
        nomint4(1,ii) = (1/4)*(sum(nomint(1,ii:ii+3)));
        nomint8(1,ii) = (1/8)*(sum(nomint(1,ii:ii+7)));
        nomint20(1,ii) = (1/20)*(sum(nomint(1,ii:ii+19)));
        nomint40(1,ii) = (1/40)*(sum(nomint(1,ii:ii+39)));
    end
    nomintallus = [nomint(1,1:irfplot);nomint2;nomint4;nomint8;nomint20;nomint40];
    
    figure;
    termstrcture = [3,6,12,24,60,120];
    for kk = 1:6
        subplot(3,2,kk)
        plot((1:irfplot),round(nomintallus(kk,1:irfplot),10),'b-','LineWidth',2);
        hold on;
        plot((1:irfplot),0*ones(irfplot,1),'r-','LineWidth',2);
        xlim([1 irfplot]);
        ylim([min(0,min(nomintallus(kk,1:irfplot))) max(0,max(nomintallus(kk,1:irfplot)))]); 
        title([num2str(termstrcture(kk)) ' Months Nom. Int. - US'],'FontSize',25);
        x12 = get(gca,'XAxis');
        set(x12,'FontSize',22);
        y12 = get(gca,'YAxis');
        set(y12,'FontSize',22);
    end
    
    % Nominal term structure - EA
    nomint = irfmat(oo_.dr.inv_order_var(17),:);
    nomint2 = zeros(1,20);
    nomint4 = zeros(1,20);
    nomint8 = zeros(1,20);
    nomint20 = zeros(1,20);
    nomint40 = zeros(1,20);
    for ii = 1:min(irfplot,irfpers-40)
        nomint2(1,ii) = (1/2)*(sum(nomint(1,ii:ii+1)));
        nomint4(1,ii) = (1/4)*(sum(nomint(1,ii:ii+3)));
        nomint8(1,ii) = (1/8)*(sum(nomint(1,ii:ii+7)));
        nomint20(1,ii) = (1/20)*(sum(nomint(1,ii:ii+19)));
        nomint40(1,ii) = (1/40)*(sum(nomint(1,ii:ii+39)));
    end
    nomintalleu = [nomint(1,1:irfplot);nomint2;nomint4;nomint8;nomint20;nomint40];

    figure;
    termstrcture = [3,6,12,24,60,120];
    for kk = 1:6
        subplot(3,2,kk)
        plot((1:irfplot),round(nomintalleu(kk,1:irfplot),10),'b-','LineWidth',2);
        hold on;
        plot((1:irfplot),0*ones(irfplot,1),'r-','LineWidth',2);
        xlim([1 irfplot]);
        ylim([min(0,min(nomintalleu(kk,1:irfplot))) max(0,max(nomintalleu(kk,1:irfplot)))]); 
        title([num2str(termstrcture(kk)) ' Months Nom. Int. - EA'],'FontSize',25);
        x12 = get(gca,'XAxis');
        set(x12,'FontSize',22);
        y12 = get(gca,'YAxis');
        set(y12,'FontSize',22);
    end
    
else
    
    dr = oo_.dr; % directory file

    % varobs = {'r';'rstar'}; % observed variables, as declared in the mod file (varobs in Dynare)
    % varobs = {'infl';'r';'inflstar';'rstar'}; % observed variables, as declared in the mod file (varobs in Dynare)
    varobs = {'infl';'r';'infltarg';'inflstar';'rstar';'infltargstar'}; % observed variables, as declared in the mod file (varobs in Dynare)
    % varobs = {'rr';'infl';'r';'rrstar';'inflstar';'rstar'}; % observed variables, as declared in the mod file (varobs in Dynare)
    % varobs = {'rr';'infl';'r';'infltarg';'rrstar';'inflstar';'rstar';'infltargstar'}; % observed variables, as declared in the mod file (varobs in Dynare)
    % varobs = {'ytilde';'rr';'infl';'r';'ytildestar';'rrstar';'inflstar';'rstar'}; % observed variables, as declared in the mod file (varobs in Dynare)
    OBS = [];
    for i = 1:rows(varobs)
        OBS = [OBS find(strcmp(deblank(varobs(i,:)),cellstr(M_.endo_names)))];
    end
    NOBS = length(OBS); % number of observed variables

    % Note: Dynare processes the model in the form
    %
    % A0*E_t[y(t+1)]+A1*y(t) = A2*y(t-1)+psi*eps(t)
    %
    % which needs to be transformed into the form used by Pearlman et al. 
    % 
    % [eps(t+1)' s(t)' x(t)' E_t[x(t+1)]']' = G*[eps(t)' s(t-1)' x(t-1)' x(t)]'+impact*eps(t)
    %
    % to obtain the solution under partial information. This is done through
    % Sim's Gensys package. The resulting system satisfies
    %
    % nmat*[E_t[s(t)]' E_t[x(t)]']'+E_t[x(t+1)] = 0 and y(t) = [TT1 TT2]*[s(t)' x(t)']'
    % 
    % which are used to generate irfs.

    G = dr.PI_ghx;
    impact = dr.PI_ghu;
    nmat = dr.PI_nmat; 
    NX = M_.exo_nbr; % number of exogenous shock variables (varexo in Dynare)
    NY = M_.endo_nbr; % number of endogenous variables (var in Dynare)
    ss = size(G,1); % dimension of the augmented state space system
    pd = ss-size(nmat,1); % number of predetermined variables in the augmented state space system
    FL_RANK = dr.PI_FL_RANK; % number of forward-looking variables in the augmented state space system

    VV = M_.H; % variance-covariance of observation errors (this is zero for what we do) 
    SDX = M_.Sigma_e^0.5; % standard deviation of exogenous shock variables (varexo in Dynare)
    MM = impact*SDX; 
    MM1 = MM(1:ss-FL_RANK,:);
    U11 = MM1*MM1'; % variance-covariance of shocks to predetermined variables
    U22 = 0; % assume no shocks to expectations of future variables

    LL = sparse(1:NOBS,OBS,ones(NOBS,1),NY,NY); % selection matrix for observable variables
    L1 = LL*dr.PI_TT1; % mapping from s(t) to observables
    L2 = LL*dr.PI_TT2; % mapping from x(t) to observables

    Gs = G(NX+1:ss-2*FL_RANK,:); % coefficients for s(t) in G
    KK1 = L1*Gs;
    K1 = KK1(:,1:ss-FL_RANK); % mapping from eps(t), s(t-1), and x(t-1) to observables
    K2 = KK1(:,ss-FL_RANK+1:ss)+L2; % mapping from x(t) to observables

    % Note: Refer to page 5 of Joe Pearlman's manual for the explanation

    % Relevant matrices
    A11 = G(1:pd,1:pd);
    A22 = G(pd+1:end,pd+1:end);
    A12 = G(1:pd,pd+1:end);
    A21 = G(pd+1:end,1:pd);
    BB = A12*inv(A22);
    FF = K2*inv(A22);
    QQ = BB*U22*BB'+U11;
    UFT = U22*FF';
    CCCC = A11-A12*nmat; % F in the manual 
    AA = A11-BB*A21; % A in the manual 
    EE = K1-K2*nmat; % E in the manual 
    DD = K1-FF*A21; % H in the manual
    RR = FF*UFT+VV; % This is zero for what we do
    if ~any(RR)
        % Adding measurement errors facilitates computation for the solver we use
        RR = eye(size(RR,1))*1.0e-6;
    end
    SS = BB*UFT;

    % Solving the Riccati equation
    AQDS = AA*QQ*DD'+SS;
    DQDR = DD*QQ*DD'+RR;
    I_DQDR = inv(DQDR);
    AQDQ = AQDS*I_DQDR;
    ff = AA-AQDQ*DD;
    hh = AA*QQ*AA'-AQDQ*AQDS';
    rr = DD*QQ*DD'+RR;
    ZSIG0 = disc_riccati_fast(ff,DD,rr,hh); % solver 
    PP = ZSIG0+QQ; % solution (P in the manual)

    % Augmented solution system (consisting of the solutions for z and ztilde in the manual)
    DPDR = DD*PP*DD'+RR;
    I_DPDR = inv(DPDR);
    PDIDPDRD = PP*DD'*I_DPDR*DD;
    GG = [CCCC (AA-CCCC)*(eye(ss-FL_RANK)-PDIDPDRD); zeros(ss-FL_RANK) AA*(eye(ss-FL_RANK)-PDIDPDRD)];
    imp = [impact(1:ss-FL_RANK,:); impact(1:ss-FL_RANK,:)];
    irfst = zeros(size(GG,1),irfpers+1); % stores irfs in terms of z and ztilde
    VV = [dr.PI_TT1 dr.PI_TT2]; % maps s(t) and x(t) to y(t)
    
    stderr = diag(M_.Sigma_e^0.5);
    eps = zeros(NX,irfpers+1); % Note: this can be any shock sequence
    eps(shockoption,1) = stderr(shockoption,1); % one standard deviation shock
        
%     eps(shockoption,1) = 1.5*stderr(shockoption,1); % selected shock
%     eps(3,1) = stderr(3,1); % monetary policy shock (for Figure 10)

%     eps(shockoption,1) = 1.5*stderr(shockoption,1); % selected shock
%     eps(3,1) = -0.5*stderr(3,1); % monetary policy shock (for Figure 16)

    irfst(:,1) = imp*eps(:,1);
    irfmat = zeros(size(dr.PI_TT1,1),irfpers+1); % stores irfs in terms of y
    for jj = 2:irfpers+1
        irfst(:,jj) = GG*irfst(:,jj-1)+imp*eps(:,jj);
        irfmat(:,jj-1) = VV*irfst(NX+1:ss-FL_RANK,jj);
    end

    figure;
    varnum = [1 3 5 6 12 14 16 17]'; % ytilde y infl r (in the declaration order)
    for kk = 1:8
        subplot(3,3,kk)
        plot((1:irfplot),round(irfmat(varnum(kk),1:irfplot),10),'b-','LineWidth',2);
        hold on;
        plot((1:irfplot),0*ones(irfplot,1),'r-','LineWidth',2);
        xlim([1 irfplot]);
        ylim([min(0,min(irfmat(varnum(kk),1:irfplot))) max(0,max(irfmat(varnum(kk),1:irfplot)))]); 
        title([deblank(M_.endo_names(varnum(kk),:))],'FontSize',25);
        x12 = get(gca,'XAxis');
        set(x12,'FontSize',22);
        y12 = get(gca,'YAxis');
        set(y12,'FontSize',22);
    end

    tot = irfmat(3,1:irfplot)-irfmat(14,1:irfplot); % terms of trade
    domprice = cumsum(irfmat(5,1:irfplot)); % initial price level normalized to zero
    forprice = cumsum(irfmat(16,1:irfplot)); % initial price level normalized to zero
    nomexc = tot+domprice-forprice; % nominal exchange rate
    
    subplot(3,3,9)
    plot((1:irfplot),round(nomexc,10),'b-','LineWidth',2);
    hold on;
    plot((1:irfplot),0*ones(irfplot,1),'r-','LineWidth',2);
    xlim([1 irfplot]);
    ylim([min(0,min(nomexc)) max(0,max(nomexc))]); 
    title('e','FontSize',25);
    x12 = get(gca,'XAxis');
    set(x12,'FontSize',22);
    y12 = get(gca,'YAxis');
    set(y12,'FontSize',22);
    
    % Nominal term structure - US
    nomint = irfmat(6,:);
    nomint2 = zeros(1,20);
    nomint4 = zeros(1,20);
    nomint8 = zeros(1,20);
    nomint20 = zeros(1,20);
    nomint40 = zeros(1,20);
    for ii = 1:min(irfplot,irfpers-40)
        nomint2(1,ii) = (1/2)*(sum(nomint(1,ii:ii+1)));
        nomint4(1,ii) = (1/4)*(sum(nomint(1,ii:ii+3)));
        nomint8(1,ii) = (1/8)*(sum(nomint(1,ii:ii+7)));
        nomint20(1,ii) = (1/20)*(sum(nomint(1,ii:ii+19)));
        nomint40(1,ii) = (1/40)*(sum(nomint(1,ii:ii+39)));
    end
    nomintallus = [nomint(1,1:irfplot);nomint2;nomint4;nomint8;nomint20;nomint40];
    yield=[nomint(1,1);nomint2(1,1);nomint4(1,1);nomint8(1,1);nomint20(1,1);nomint40(1,1)];
    
    figure;
    termstrcture = [3,6,12,24,60,120];
    for kk = 1:6
        subplot(3,2,kk)
        plot((1:irfplot),round(nomintallus(kk,1:irfplot),10),'b-','LineWidth',2);
        hold on;
        plot((1:irfplot),0*ones(irfplot,1),'r-','LineWidth',2);
        xlim([1 irfplot]);
        ylim([min(0,min(nomintallus(kk,1:irfplot))) max(0,max(nomintallus(kk,1:irfplot)))]); 
        title([num2str(termstrcture(kk)) ' Months Nom. Int. - US'],'FontSize',25);
        x12 = get(gca,'XAxis');
        set(x12,'FontSize',22);
        y12 = get(gca,'YAxis');
        set(y12,'FontSize',22);
    end
    
    % Nominal term structure - EU
    nomint = irfmat(17,:);
    nomint2 = zeros(1,20);
    nomint4 = zeros(1,20);
    nomint8 = zeros(1,20);
    nomint20 = zeros(1,20);
    nomint40 = zeros(1,20);
    for ii = 1:min(irfplot,irfpers-40)
        nomint2(1,ii) = (1/2)*(sum(nomint(1,ii:ii+1)));
        nomint4(1,ii) = (1/4)*(sum(nomint(1,ii:ii+3)));
        nomint8(1,ii) = (1/8)*(sum(nomint(1,ii:ii+7)));
        nomint20(1,ii) = (1/20)*(sum(nomint(1,ii:ii+19)));
        nomint40(1,ii) = (1/40)*(sum(nomint(1,ii:ii+39)));
    end
    nomintalleu = [nomint(1,1:irfplot);nomint2;nomint4;nomint8;nomint20;nomint40];

    figure;
    termstrcture = [3,6,12,24,60,120];
    for kk = 1:6
        subplot(3,2,kk)
        plot((1:irfplot),round(nomintalleu(kk,1:irfplot),10),'b-','LineWidth',2);
        hold on;
        plot((1:irfplot),0*ones(irfplot,1),'r-','LineWidth',2);
        xlim([1 irfplot]);
        ylim([min(0,min(nomintalleu(kk,1:irfplot))) max(0,max(nomintalleu(kk,1:irfplot)))]); 
        title([num2str(termstrcture(kk)) ' Months Nom. Int. - EA'],'FontSize',25);
        x12 = get(gca,'XAxis');
        set(x12,'FontSize',22);
        y12 = get(gca,'YAxis');
        set(y12,'FontSize',22);
    end
    
%     figure;
%     subplot(3,3,1)
%     plot((1:6)',10000*yield,'b-','LineWidth',2);
%     hold on;
%     plot((1:6)',0*ones(6,1),'r-','LineWidth',2);
%     xlim([1 6]);
%     xticks((1:6));
%     xticklabels({'3','6','12','24','60','120'});
%     title('US yields (in bps)','FontSize',25);
%     x12 = get(gca,'XAxis');
%     set(x12,'FontSize',22);
%     y12 = get(gca,'YAxis');
%     set(y12,'FontSize',22);
%     legend('Inf and int rate shocks','FontSize',20);
%     subplot(3,3,2)
%     plot((1:irfplot),100*nomexc,'b-','LineWidth',2);
%     hold on;
%     plot((1:irfplot),0*ones(irfplot,1),'r-','LineWidth',2);
%     xlim([1 irfplot]);
%     ylim([min(0,min(100*nomexc)) max(0,max(100*nomexc))]); 
%     title('USD/EUR (in %)','FontSize',25);
%     x12 = get(gca,'XAxis');
%     set(x12,'FontSize',22);
%     y12 = get(gca,'YAxis');
%     set(y12,'FontSize',22);

end