% A replication file for "Monetary Policy Surprises and Exchange Rate
% Behavior" by Gurkaynak, Kara, Kisacikoglu and Lee.

% This m file generates target moments for structural estimation.

clear;
clc;

% Load data 

clc;
clear; 

data = xlsread('macro_data.xls'); 
zlbdum = 1; 
if zlbdum == 1 
    data = data(1:59,:); 
end

eu_rgdp = data(:,1); % EA real gdp, seasonally adjusted, chained 2012
us_rgdp = data(:,2); % US real gdp, seasonally adjusted, chained 2010
eu_int = 0.01*data(:,3); % main refinancing rate
us_int = 0.01*data(:,4); % effective Fed funds rate
eu_price = data(:,5); % EA harmoized price index, seasonally adjusted
us_price = data(:,6); % US consumer price index, seasonally adjusted

%-----------------------------------------------------------------------
% Calculate moments: standard deviation, correlation, first order
% autocorrelation
% Detrend output and inflation using HP filter. Demean policy rates
%-------------------------------------------------------------------------

% Policy rates

ii = [eu_int us_int]; 
ii = rmmissing(ii); 
iisizediff = size(data,1)-size(ii,1);
eu_int_raw = ii(:,1);
us_int_raw = ii(:,2);

[T,~] = size(ii); 
dm_int_eu = eu_int_raw-repmat(mean(eu_int_raw),T,1); 
dm_int_us = us_int_raw-repmat(mean(us_int_raw),T,1); 

std_int_eu = std(dm_int_eu); 
std_int_us = std(dm_int_us); 
autocorrmat_int_eu = corrcoef(dm_int_eu(2:end,1),dm_int_eu(1:end-1,1));
autocorr_int_eu = autocorrmat_int_eu(1,2);
autocorrmat_int_us = corrcoef(dm_int_us(2:end,1),dm_int_us(1:end-1,1)); 
autocorr_int_us = autocorrmat_int_us(1,2); 

% Output

yy = [eu_rgdp us_rgdp]; 
yy = yy(iisizediff+1:end,:); 
eu_rgdp_raw = yy(:,1);
us_rgdp_raw = yy(:,2);

[~,c_rgdp_eu] = hpfilter(log(eu_rgdp_raw),1600); 
[~,c_rgdp_us] = hpfilter(log(us_rgdp_raw),1600); 

std_rgdp_eu = std(c_rgdp_eu);
std_rgdp_us = std(c_rgdp_us);
autocorrmat_rgdp_eu = corrcoef(c_rgdp_eu(2:end,1),c_rgdp_eu(1:end-1,1));
autocorr_rgdp_eu = autocorrmat_rgdp_eu(1,2);
autocorrmat_rgdp_us = corrcoef(c_rgdp_us(2:end,1),c_rgdp_us(1:end-1,1)); 
autocorr_rgdp_us = autocorrmat_rgdp_us(1,2);

% Inflation (year-on-year)

pp = [eu_price us_price]; 
pp = pp(iisizediff+1-4:end,:); 
eu_price_raw = pp(:,1); 
us_price_raw = pp(:,2);

[T,~] = size(pp);
for t = 1:T-4
eu_inf_raw(t,1) = log(eu_price_raw(t+4,:)/eu_price_raw(t,:)); 
us_inf_raw(t,1) = log(us_price_raw(t+4,:)/us_price_raw(t,:)); 
end

[~,c_inf_eu] = hpfilter(eu_inf_raw,1600); 
[~,c_inf_us] = hpfilter(us_inf_raw,1600);

% dm_inf_eu = eu_inf_raw-repmat(mean(eu_inf_raw),T-4,1);
% c_inf_eu = dm_inf_eu;
% dm_inf_us = us_inf_raw-repmat(mean(us_inf_raw),T-4,1);
% c_inf_us = dm_inf_us;

std_inf_eu = std(c_inf_eu); 
std_inf_us = std(c_inf_us); 
autocorrmat_inf_eu = corrcoef(c_inf_eu(2:end,1),c_inf_eu(1:end-1,1));
autocorr_inf_eu = autocorrmat_inf_eu(1,2);
autocorrmat_inf_us = corrcoef(c_inf_us(2:end,1),c_inf_us(1:end-1,1));
autocorr_inf_us = autocorrmat_inf_us(1,2);

% Standard deviations

disp('Standard deviations')
disp(' ')
disp('EA policy rate')
disp(std_int_eu)
disp('US policy rate')
disp(std_int_us)
disp('EA real GDP')
disp(std_rgdp_eu)
disp('US real GDP')
disp(std_rgdp_us)
disp('EA inflation')
disp(std_inf_eu)
disp('US inflation')
disp(std_inf_us)

% Correlations

filtered_data = [dm_int_eu dm_int_us c_rgdp_eu c_rgdp_us c_inf_eu c_inf_us];
corrmat = corrcoef(filtered_data);
disp('----------')
disp('Correlations')
disp(' ')
disp('Correlation between EA and US policy rate')
disp(corrmat(1,2))
disp('Correlation between EA policy rate and real GDP')
disp(corrmat(1,3))
disp('Correlation between EA policy rate and US real GDP')
disp(corrmat(1,4))
disp('Correlation between EA policy rate and inflation')
disp(corrmat(1,5))
disp('Correlation between EA policy rate and US inflation')
disp(corrmat(1,6))
disp('Correlation between US policy rate and EA real GDP')
disp(corrmat(2,3))
disp('Correlation between US policy rate and real GDP')
disp(corrmat(2,4))
disp('Correlation between US policy rate and EA inflation')
disp(corrmat(2,5))
disp('Correlation between US policy rate and inflation')
disp(corrmat(2,6))
disp('Correlation between EA and US real GDP')
disp(corrmat(3,4))
disp('Correlation between EA real GDP and inflation')
disp(corrmat(3,5))
disp('Correlation between EA real GDP and US inflation')
disp(corrmat(3,6))
disp('Correlation between US real GDP and EA inflation')
disp(corrmat(4,5))
disp('Correlation between US real GDP and inflation')
disp(corrmat(4,6))
disp('Correlation between EA and US inflation')
disp(corrmat(5,6))

% Autocorrelations

disp('----------')
disp('Autocorrelations')
disp(' ')
disp('EA policy rate first order autocorrelation')
disp(autocorr_int_eu)
disp('US policy rate first order autocorrelation')
disp(autocorr_int_us)
disp('EA real GDP first order autocorrelation')
disp(autocorr_rgdp_eu)
disp('US real GDP first order autocorrelation')
disp(autocorr_rgdp_us)
disp('EA inflation first order autocorrelation')
disp(autocorr_inf_eu)
disp('US inflation first order autocorrelation')
disp(autocorr_inf_us)

% A vector of target moments

target_moments = zeros(27,1);
target_moments(1,1) = std_int_eu;
target_moments(2,1) = std_int_us;
target_moments(3,1) = std_rgdp_eu;
target_moments(4,1) = std_rgdp_us;
target_moments(5,1) = std_inf_eu;
target_moments(6,1) = std_inf_us;
target_moments(7,1) = corrmat(1,2);
target_moments(8,1) = corrmat(1,3);
target_moments(9,1) = corrmat(1,4);
target_moments(10,1) = corrmat(1,5);
target_moments(11,1) = corrmat(1,6);
target_moments(12,1) = corrmat(2,3);
target_moments(13,1) = corrmat(2,4);
target_moments(14,1) = corrmat(2,5);
target_moments(15,1) = corrmat(2,6);
target_moments(16,1) = corrmat(3,4);
target_moments(17,1) = corrmat(3,5);
target_moments(18,1) = corrmat(3,6);
target_moments(19,1) = corrmat(4,5);
target_moments(20,1) = corrmat(4,6);
target_moments(21,1) = corrmat(5,6);
target_moments(22,1) = autocorr_int_eu;
target_moments(23,1) = autocorr_int_us;
target_moments(24,1) = autocorr_rgdp_eu;
target_moments(25,1) = autocorr_rgdp_us;
target_moments(26,1) = autocorr_inf_eu;
target_moments(27,1) = autocorr_inf_us;

save target_moments.mat target_moments;
